<!DOCTYPE html>
<html>
<head>
    <title>Swagger explorer</title>

    <meta name="viewport" content="width=device-width, initial-scale=1.0">

    <!--[if lt IE 9]>
    <script src="http://html5shim.googlecode.com/svn/trunk/html5.js"></script><![endif]-->
    <!--style type="text/css"></style-->

    <!-- link href="https://cdn.rawgit.com/jensoleg/swagger-ui/master/src/main/html/css/index.css" rel="stylesheet" type="text/css"/>
    <link href='https://cdn.rawgit.com/jensoleg/swagger-ui/master/src/main/html/css/standalone.css' rel='stylesheet' type="text/css" />
    <link href='https://cdn.rawgit.com/jensoleg/swagger-ui/master/src/main/html/css/api-explorer.css' rel='stylesheet' type='text/css' type="text/css" />
    <link href='https://cdn.rawgit.com/jensoleg/swagger-ui/master/src/main/html/css/screen.css' media='screen' rel='stylesheet' type='text/css' type="text/css" -->


    <link href="https://rawcdn.githack.com/jensoleg/swagger-ui/master/dist/css/index.css" rel="stylesheet"/>
    <link href='https://rawcdn.githack.com/jensoleg/swagger-ui/master/dist/css/standalone.css' rel='stylesheet'/>
    <link href='https://rawcdn.githack.com/jensoleg/swagger-ui/master/dist/css/api-explorer.css' rel='stylesheet' type='text/css'/>
    <link href='https://rawcdn.githack.com/jensoleg/swagger-ui/master/dist/css/screen.css' media='screen' rel='stylesheet' type='text/css'/>
    <style>
        .swagger-section .parameter-item .param-property
        {
            padding-left: 300px;
        }
        .swagger-section .parameter-item .param-property:before
        {
            width: 300px;
        }
        .tooltip, .tooltip-inner
        {
            max-width: 400px;
        }
        .swagger-section .parameter-item .param-property[data-label="body"]{
            padding-left: 10px;
            padding-top: 30px;
        }
        .swagger-section .parameter-item .param-property[data-label="body"]:before{
            top:6px
        }
        #okta h2{
            margin: 0;
            color: #5c666f;
            font-size: 13px;
            margin-bottom: 12px;
            margin-top: 15px;
            text-transform: uppercase;
            letter-spacing: .08em;
            font-weight: 700;
        }
        #okta .auth-footer, #form18 > div.o-form-content.o-form-theme.clearfix > div.o-form-fieldset-container > div.o-form-fieldset.o-form-label-top.margin-btm-0{
            display: none
        }
        ul.json-dict li{
            margin:3px 0
        }
    </style>



    <script src='https://cdnjs.cloudflare.com/ajax/libs/jquery/1.8.0/jquery.js' type='text/javascript'></script>

    <script src='https://cdn.rawgit.com/dblate/jquery.json-editor/2813894b/dist/jquery.json-editor.min.js' type='text/javascript'></script>
    <script>
        JSONPrettyEditor = JsonEditor;
    </script>

    <script src='https://cdn.rawgit.com/jensoleg/swagger-ui/master/lib/jquery.slideto.min.js' type='text/javascript'></script>
    <script src='https://cdn.rawgit.com/jensoleg/swagger-ui/master/lib/jquery.wiggle.min.js' type='text/javascript'></script>
    <script src='https://cdn.rawgit.com/jensoleg/swagger-ui/master/lib/jquery.ba-bbq.min.js' type='text/javascript'></script>
    <script src='https://cdn.rawgit.com/jensoleg/swagger-ui/master/lib/handlebars-2.0.0.js' type='text/javascript'></script>
    <script src='https://cdn.rawgit.com/jensoleg/swagger-ui/master/lib/underscore-min.js' type='text/javascript'></script>
    <script src='https://cdn.rawgit.com/jensoleg/swagger-ui/master/lib/backbone-min.js' type='text/javascript'></script>

    <script src='https://cdn.rawgit.com/jensoleg/swagger-ui/master/dist/swagger-ui.js' type='text/javascript'></script>
    <script src='https://cdn.rawgit.com/jensoleg/swagger-ui/master/lib/jsoneditor.js' type='text/javascript'></script>
    <script src='https://cdn.rawgit.com/jensoleg/swagger-ui/master/lib/highlight.7.3.pack.js' type='text/javascript'></script>
    <script src='https://cdn.rawgit.com/jensoleg/swagger-ui/master/lib/marked.js' type='text/javascript'></script>
    <script src='https://cdn.rawgit.com/jensoleg/swagger-ui/master/lib/swagger-oauth.js' type='text/javascript'></script>
    <script src='https://cdn.rawgit.com/jensoleg/swagger-ui/master/lib/bootstrap.min.js' type='text/javascript'></script>


    <script
    src="https://ok1static.oktacdn.com/assets/js/sdk/okta-signin-widget/2.6.0/js/okta-sign-in.min.js"
    type="text/javascript"></script>

    <script  type='text/javascript'>
        $( function(){
            var old = window.jQuery.fn.init;
            clean = function ( selector, context, rootjQuery ){
                if(typeof selector == "string" && selector[0] != '<'){
                    selector = decodeURIComponent(selector).replace(new RegExp("%2F", 'g'),"\\/").replace(new RegExp("/", 'g'),"\\/")
                }
                return new old( selector, context, rootjQuery )
            }
            clean.prototype = old.prototype
            window.jQuery.fn.init = clean
        })

    </script>

    <script type="text/javascript">
        jQuery.browser = jQuery.browser || {};
        (function () {
            jQuery.browser.msie = jQuery.browser.msie || false;
            jQuery.browser.version = jQuery.browser.version || 0;
            if (navigator.userAgent.match(/MSIE ([0-9]+)\./)) {
                jQuery.browser.msie = true;
                jQuery.browser.version = RegExp.$1;
            }
        })();
    </script>

    <script type="text/javascript">
        $(function () {
            var url = window.location.search.match(/url=([^&]+)/);
            if (url && url.length > 1) {
                url = decodeURIComponent(url[1]);
            } else {
                url = window.location.toString().replace(/\/[^\/]*(#.*)?$/, '/swagger.json');
            }
            window.swaggerUi = new SwaggerUi({
                url: url,
                dom_id: "swagger-ui-container",
                supportedSubmitMethods: ['get', 'post', 'put', 'delete', 'patch'],
                onComplete: function (swaggerApi, swaggerUi) {
                    if (typeof initOAuth == "function") {
                        initOAuth({
                            clientId: "ffe7748a-3a3f-4860-a02a-42ab08e4fde2",
                            realm: "realm",
                            appName: "Swagger"
                        });
                    }
                    $('pre code').each(function (i, e) {
                        hljs.highlightBlock(e)
                    });
                    if (swaggerUi.options.url) {
                        $('#input_baseUrl').val(swaggerUi.options.url);
                    }
                    if (swaggerUi.options.apiKey) {
                        $('#input_apiKey').val(swaggerUi.options.apiKey);
                    }
                    $(".path>a").each(function(){
                        $(this).attr("data-toggle", "tooltip")
                        $(this).attr("data-placement", "bottom")
                        $(this).attr("data-original-title", $(this).text())
                    });
                    $("[data-toggle='tooltip']").tooltip();


                    addApiKeyAuthorization();

                    // The following enable a get response to be used for a put in one click
                    $(".get").not('[id*="GET_ALL"]').find(".modal-body").append("<br /><input class=\"btn btn-primary\" name=\"use_for_put\" type=\"submit\" value=\"Use for PUT\" data-target=\"#get_clients-modal-request\">")
                    $(".btn[name='use_for_put']").on("click", function(e){
                        var modal_content = $(this).closest(".modal-content")
                        var response_body = modal_content.find(".response_body").text()
                        var response_headers = JSON.parse(modal_content.find(".response_headers").text())
                        var get = $(this).closest(".get")
                        var put = $(this).closest("ul.endpoints").find("li.put")

                        if("etag" in response_headers){
                            put.find("input[name='etag']").val(response_headers["etag"])
                        }
                        get.find("input[type='text']").each(function(i, input)
                        {
                            var mirror_input = put.find("input[type='text'][name='" + $(input).attr("name")+"']")
                            mirror_input.val($(input).val())
                            mirror_input.trigger("change")
                        })

                        var put_body=put.find("textarea[name='body']")
                        put_body.val(response_body)
                        put_body.trigger("change")
                        modal_content.find(".close").trigger("click")
                        $("#resources_nav").find("[data-endpoint='"+put[0].id+"']").trigger("click")
                        return false;
                    })

                    // The following enable body to be edited with json highlighting
                    $(".snippet").parent().parent().off("mousedown")
                    $("textarea[name='body']").each(function(i, editor){
                        $(editor).after('<div id="json_editor_'+i+'"></div>')
                        var jEditor = new JSONPrettyEditor('#json_editor_'+i, "")
                        var jEditor_div = $('#json_editor_'+i).on("keyup", function(){
                            $(editor).val(JSON.stringify(jEditor.get()))
                        })
                        $(editor).closest("form").on("submit", function(){
                            $(editor).val(JSON.stringify(jEditor.get()))
                        })
                        $(editor).on("change", function(){
                            try{
                                jEditor.load(JSON.parse($(editor).val()))
                            }
                            catch(ex){
                                console.log(ex)
                            }
                        })
                        $(editor).trigger("change")
                        $(editor).hide()
                    })


                    // The following enable okta login
                    {% if okta is defined %}
                    $(".token-generator .scope-selector").remove()
                    $(".token-generator").append('<div id="okta"></div>')
                    var signIn = new OktaSignIn({
                        baseUrl: '{{okta.baseUrl}}',
                        clientId:"{{okta.clientId}}",
                        redirectUri: "{{okta.redirectUri}}",
                        authParams: {
                            responseType: ['token'],
                            {%- if okta.auth_issuer is defined %}
                            issuer: "{{okta.auth_issuer}}"
                            {%- endif %}
                        }
                    });

                    signIn.renderEl({
                        el: '#okta'
                    }, function success(res) {
                        if (res.status === 'SUCCESS') {
                            console.log('Do something with this sessionToken',  res);
                            $('#input_apiKey').val(res[0].accessToken)
                            $("#okta").hide()
                            addApiKeyAuthorization();
                        } else {
                            console.log('Auth failed:',res);
                        }
                    });
                    {% endif %}
                },
                onFailure: function (data) {
                    log("Unable to Load SwaggerUI");
                },
                docExpansion: "none",
                sorter: "alpha"
            });
            function addApiKeyAuthorization() {
                var key = encodeURIComponent($('#input_apiKey')[0].value);
                if (key && key.trim() != "") {
                    var apiKeyAuth = new SwaggerClient.ApiKeyAuthorization("Authorization", "Bearer " + key, "header");
                    window.swaggerUi.api.clientAuthorizations.add("key", apiKeyAuth);
                    log("added key " + key);
                }
            }
            $('#input_apiKey').change(addApiKeyAuthorization);
            // if you have an apiKey you would like to pre-populate on the page for demonstration purposes...
            /*
             var apiKey = "myApiKeyXXXX123456789";
             $('#input_apiKey').val(apiKey);
             */
            window.swaggerUi.load();
            function log() {
                if ('console' in window) {
                    console.log.apply(console, arguments);
                }
            }
        });
    </script>

    <script type="text/javascript">
        $(function () {
            $(window).scroll(function () {
                var sticky = $(".sticky-nav");
                i(sticky);
                r(sticky);
                function n() {
                    return window.matchMedia("(min-width: 992px)").matches
                }
                function e() {
                    n() ? sticky.parents(".sticky-nav-placeholder").removeAttr("style") : sticky.parents(".sticky-nav-placeholder").css("min-height", sticky.outerHeight())
                }
                function i(n) {
                    n.hasClass("fixed") || (navOffset = n.offset().top);
                    e();
                    $(window).scrollTop() > navOffset ? $(".modal.in").length || n.addClass("fixed") : n.removeClass("fixed")
                }
                function r(e) {
                    function i() {
                        var i = $(window).scrollTop(), r = e.parents(".sticky-nav");
                        return r.hasClass("fixed") && !n() && (i = i + r.outerHeight() + 40), i
                    }
                    function r(e) {
                        var t = o.next("[data-endpoint]"), n = o.prev("[data-endpoint]");
                        return "next" === e ? t.length ? t : o.parent().next().find("[data-endpoint]").first() : "prev" === e ? n.length ? n : o.parent().prev().find("[data-endpoint]").last() : []
                    }
                    var nav = e.find("[data-navigator]");
                    if (nav.find("[data-endpoint][data-selected]").length) {
                        var o = nav.find("[data-endpoint][data-selected]"),
                            a = $("#" + o.attr("data-endpoint")),
                            s = a.offset().top,
                            l = (s + a.outerHeight(), r("next")),
                            u = r("prev");
                        if (l.length) {
                            {
                                var d = $("#" + l.attr("data-endpoint")), f = d.offset().top;
                                f + d.outerHeight()
                            }
                            i() >= f && c(l)
                        }
                        if (u.length) {
                            var p = $("#" + u.attr("data-endpoint")),
                            g = u.offset().top;
                            v = (g + p.outerHeight(), 100);
                            i() < s - v && c(u)
                        }
                    }
                }
                function s() {
                    var e = $(".sticky-nav [data-navigator]"),
                        n = e.find("[data-endpoint]").first();
                    n.attr("data-selected", "");
                    u.find("[data-selected-value]").html(n.text())
                }
                function c(e) {
                    {
                        var n = $(".sticky-nav [data-navigator]");
                        $("#" + e.attr("data-endpoint"))
                    }
                    n.find("[data-resource]").removeClass("active");
                    n.find("[data-selected]").removeAttr("data-selected");
                    e.closest("[data-resource]").addClass("active");
                    e.attr("data-selected", "");
                    sticky.find("[data-selected-value]").html(e.text())
                }
            });
        });
    </script>


</head>

<body class="page-docs" style="zoom: 1;">
<header class="site-header">
    <nav role="navigation" class="navbar navbar-default">
        <div class="container">
            <div class="navbar-header">
                <button type="button" data-toggle="collapse" data-target="#navbar-collapse" class="navbar-toggle"><span
                        class="sr-only">Toggle navigation</span><span class="icon-bar"></span><span
                        class="icon-bar"></span><span class="icon-bar"></span></button>
                <h1 class="navbar-brand"><a href="http://swagger.io"><span>swagger explorer</span></a></h1>
            </div>
            <div id="navbar-collapse" class="collapse navbar-collapse">
                <ul class="nav navbar-nav navbar-left">
                    <li class="li-why"><a href="http://swagger.io" style="font-size: 25px; padding-left: 0px">Swagger
                        explorer</a></li>
                    <li><input id="input_apiKey" type="hidden" /></li>

                </ul>
            </div>
        </div>
    </nav>
</header>

<section class="content">
    <div id="api2-explorer">
        <div class="swagger-section page-docs" style="zoom: 1">
            <div class="main-section">
                <div id="swagger-ui-container" class="swagger-ui-wrap">
                </div>
            </div>
        </div>
    </div>
</section>
</body>
</html>